/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2015                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __VARIABLE_EVALUATION_CPP
#define __VARIABLE_EVALUATION_CPP 0

#include "CubeVariableEvaluation.h"
#include "CubeServices.h"

using namespace cube;
using namespace std;


VariableEvaluation::~VariableEvaluation()
{
    delete index;
}


double
VariableEvaluation::eval()
{
    return memory->get( variable, index->eval(), kind  );
}



double
VariableEvaluation::eval( Cnode* cnode, CalculationFlavour cf, Sysres* sr, CalculationFlavour tf  )
{
    return memory->get( variable, index->eval( cnode, cf, sr, tf ), kind  );
};



double
VariableEvaluation::eval( Cnode* cnode, CalculationFlavour cf )
{
    return memory->get( variable, index->eval( cnode, cf ), kind  );
};


double
VariableEvaluation::eval( double arg1, double arg2 )
{
    return memory->get( variable, index->eval( arg1, arg2 ), kind  );
};



string
VariableEvaluation::strEval()
{
    return memory->get_as_string( variable, index->eval(), kind  );
}


double*
VariableEvaluation::eval_row( Cnode*             cnode,
                              CalculationFlavour cf )
{
    double* _tmp = memory->get_row( variable, index->eval( cnode, cf ), kind  );
    if ( _tmp == NULL )
    {
        return NULL;
    }
    double* to_return = services::create_row_of_doubles( row_size );
    memcpy( to_return, _tmp, row_size * sizeof( double ) );
    return to_return;
}



#endif
